import { Component, EventEmitter, Input, OnChanges, Output } from '@angular/core';
import { UntypedFormGroup } from '@angular/forms';
import { Structure } from '../../../models/structure.model';
import { User } from '../../../models/user.model';
import { Filter } from '../../../structure-list/models/filter.model';
import { OrientationUtils } from '../../../utils/orientationUtils';
import { FiltersSteps, NeedsType, RecapsType, StructuresListSteps } from '../enums/orientation.enums';
import { MediationStepType } from '../types/orientation.types';

@Component({
  selector: 'app-orientation-structure-list',
  templateUrl: './orientation-structure-list.component.html',
})
export class OrientationStructureListComponent implements OnChanges {
  @Input() currentStep: StructuresListSteps | FiltersSteps | MediationStepType | NeedsType;
  @Input() profile: User;
  @Input() form: UntypedFormGroup;
  @Input() filters: Filter[] = [];
  @Output() validatePage = new EventEmitter<any>();

  public orientationUtils = new OrientationUtils();

  // Enum
  public StructuresListSteps = StructuresListSteps;
  public RecapsType = RecapsType;
  // Init data for form
  public selectedStructures: Structure[] = [];

  ngOnChanges(): void {
    this.validatePage.emit();
  }

  public radioChange(event: { name: string; value: boolean }): void {
    const { name, value } = event;
    this.form.get(name).setValue(value);
    this.orientationUtils.manuallySetOfPmr(this.filters, event);
    this.checkValidation();
  }

  public checkValidation(event?: any): void {
    // Reset value of selected structure in case of multiple previous
    if (this.currentStep === StructuresListSteps.address) {
      this.selectedStructures = [];
    }
    if (this.currentStep === StructuresListSteps.structureChoice) {
      this.selectedStructures = event;
      this.form.get('structureChoice').setValue(event);
    }
    if (this.currentStep === StructuresListSteps.structureOrientator) {
      this.validatePage.emit(event);
      return;
    }
    this.validatePage.emit(event);
  }
}
